
# q-expansion of newform 216.4.a.e, downloaded from the LMFDB on 05 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 216
weight = 4
poly_data = [-8, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[-3, 6], 1]]

hecke_ring_character_values = None
aps_data = [[0, 0], [0, 0], [-4, -1], [12, 1], [-1, 0], [16, -4], [28, 4], [92, 2], [-46, 4], [168, -2], [188, -5], [174, -4], [156, -10], [40, -14], [114, 8], [-76, 13], [-340, 24], [-56, -32], [176, 34], [-908, 12], [-287, 0], [-680, 32], [391, -32], [-120, -18], [-169, 8], [828, 9], [-56, -52], [-693, 56], [46, 76], [732, 70], [-164, 23], [1003, 104], [1296, -2], [1416, -100], [-2132, 31], [-1468, -113], [-764, 72], [1236, 120], [390, -144], [-3252, 19], [1197, -56], [2016, 16], [840, 196], [-499, -40], [-1092, 245], [1564, -129], [3028, 34], [-2192, 8], [-1740, -24], [-1682, -8], [3128, 50], [2878, 164], [1950, 248], [1168, -336], [3940, 56], [-2158, -132], [-560, -226], [68, 185], [-3008, -24], [-284, 116], [-1420, -178], [624, -134], [-8148, 4], [-1110, -84], [-2427, -96], [-36, 449], [1952, -134], [-5694, -56], [-2655, -304], [1434, -156], [460, -526], [3462, 24], [-8564, -193], [7628, 276], [1856, 12], [-10044, -108], [-5668, -409], [-5152, 36], [7104, -132], [9279, 128], [612, -512], [3584, 360], [-7346, -224], [5269, -264], [3780, 291], [-6884, 224], [-9652, 378], [-5569, -200], [7868, 255], [3524, -3], [-1209, -616], [10114, -488], [3448, 248], [-14571, -328], [2256, -926], [-7986, -332], [948, -771], [-6380, 288], [-6884, 788], [2556, 264], [4652, 428], [-14788, 15], [14921, -272], [-7144, 396], [-15204, 488], [-17074, 344], [15959, 456], [-2920, 1122], [6594, -104], [-2645, 416], [-19528, 264], [-3514, 880], [996, -38], [21724, -192], [7612, 345], [7172, -978], [-8016, 276], [13768, -328], [1308, 123], [12993, 48], [-12862, -572], [-3203, -112], [872, -590], [-9964, -224], [9872, -1212], [15540, -875], [27548, -64], [-20212, 272], [-7692, 1545], [5486, -1292], [2804, 1596], [-4456, -1604], [17172, -449], [1970, -40], [-22520, 168], [23849, -360], [14304, 478], [-22592, -992], [14900, -749], [14200, 434], [-2588, -726], [4608, -842], [19916, -489], [-31424, -320], [34, 184], [-3492, 268], [13286, -408], [-11004, -468], [8040, 1088], [-16828, -1696], [2850, 1952], [8908, 1696], [7172, -1190], [34576, -380], [-9116, -2094], [-8230, 2208], [36916, 3], [-42344, -468], [-14853, -888], [24660, 1019], [-24999, -880], [17064, -1404], [-11596, 641], [-13929, -992], [6408, -826], [-5934, 3096], [14428, 1673], [-44552, 252]]
